%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%

-module(alloc_SUITE).
-author('rickard.green@uab.ericsson.se').
-export([all/1]).

-export([basic/1,
	 coalesce/1,
	 threads/1,
	 realloc_copy/1,
	 bucket_index/1,
	 bucket_mask/1,
	 rbtree/1]).

-include("test_server.hrl").

-define(DEFAULT_TIMETRAP_SECS, 240).

all(doc) -> [];
all(suite) -> [basic,
	       coalesce,
	       threads,
	       realloc_copy,
	       bucket_index,
	       bucket_mask,
	       rbtree].

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                        %%
%% Testcases                                                              %%
%%                                                                        %%

basic(suite) -> [];
basic(doc) ->   [];
basic(Cfg) -> ?line drv_case(Cfg, basic).

coalesce(suite) -> [];
coalesce(doc) ->   [];
coalesce(Cfg) -> ?line drv_case(Cfg, coalesce).

threads(suite) -> [];
threads(doc) ->   [];
threads(Cfg) -> ?line drv_case(Cfg, threads).

realloc_copy(suite) -> [];
realloc_copy(doc) ->   [];
realloc_copy(Cfg) -> ?line drv_case(Cfg, realloc_copy).

bucket_index(suite) -> [];
bucket_index(doc) ->   [];
bucket_index(Cfg) -> ?line drv_case(Cfg, bucket_index).

bucket_mask(suite) -> [];
bucket_mask(doc) ->   [];
bucket_mask(Cfg) -> ?line drv_case(Cfg, bucket_mask).

rbtree(suite) -> [];
rbtree(doc) ->   [];
rbtree(Cfg) -> ?line drv_case(Cfg, rbtree).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                        %%
%% Internal functions                                                     %%
%%                                                                        %%

drv_case(Config, CaseName) ->
    drv_case(Config, CaseName, "").

drv_case(Config, CaseName, TimeTrap) when integer(TimeTrap) ->
    drv_case(Config, CaseName, "", TimeTrap);
drv_case(Config, CaseName, Command) when list(Command) ->
    drv_case(Config, CaseName, Command, ?DEFAULT_TIMETRAP_SECS).

drv_case(Config, CaseName, TimeTrap, Command) when list(Command),
						   integer(TimeTrap) ->
    drv_case(Config, CaseName, Command, TimeTrap);
drv_case(Config, CaseName, Command, TimeTrap) when list(Config),
						   atom(CaseName),
						   list(Command),
						   integer(TimeTrap) ->
    case ?t:os_type() of
	{Family, _} when Family == unix; Family == win32 ->
	    ?line run_drv_case(Config, CaseName, Command, TimeTrap);
	SkipOs ->
	    ?line {skipped,
		   lists:flatten(["Not run on "
				  | io_lib:format("~p",[SkipOs])])}
    end.

run_drv_case(Config, CaseName, Command, TimeTrap) ->
    ?line Dog = test_server:timetrap(test_server:seconds(TimeTrap)),
    ?line DataDir = ?config(data_dir,Config),
    ?line ok = erl_ddll:load_driver(DataDir, CaseName),
    ?line Port = open_port({spawn, atom_to_list(CaseName)}, []),
    ?line true = is_port(Port),
    ?line Port ! {self(), {command, Command}},
    ?line Result = receive_drv_result(Port, CaseName),
    ?line Port ! {self(), close},
    ?line receive 
	      {Port, closed} ->
		  ok
	  end,
    ?line ok = erl_ddll:unload_driver(CaseName),
    ?line test_server:timetrap_cancel(Dog),
    ?line Result.

receive_drv_result(Port, CaseName) ->
    ?line Result = receive
		       {print, Port, CaseName, Str} ->
			   ?line ?t:format("~s", [Str]),
			   ?line receive_drv_result(Port, CaseName);
		       {'EXIT', Port, Error} ->
			   ?line ?t:fail(Error);
		       {'EXIT', error, Error} ->
			   ?line ?t:fail(Error);
		       {failed, Port, CaseName, Comment} ->
			   ?line ?t:fail(Comment);
		       {skipped, Port, CaseName, Comment} ->
			   ?line {skipped, Comment};
		       {succeeded, Port, CaseName, ""} ->
			   ?line succeeded;
		       {succeeded, Port, CaseName, Comment} ->
			   ?line {comment, Comment}
		   end.
