%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
-module(evil_SUITE).

-export([all/1,init_per_testcase/2,fin_per_testcase/2,
	 heap_frag/1,heap_frag_vec/1]).

-include("test_server.hrl").

all(suite) -> [heap_frag,heap_frag_vec].

init_per_testcase(_Case, Config) ->
    ?line Dog = test_server:timetrap(?t:minutes(0.5)),
    [{watchdog, Dog}|Config].

fin_per_testcase(_Case, Config) ->
    Dog=?config(watchdog, Config),
    test_server:timetrap_cancel(Dog),
    ok.

heap_frag(Config) when is_list(Config) ->
    N = 512,
    Self = self(),
    ?line Pid = spawn_link(fun() -> appender(Self, N) end),
    receive
	{Pid,Res} ->
	    ?line Res = my_appender(N);
	Garbage ->
	    io:format("Garbage: ~p\n", [Garbage]),
	    ?line ?t:fail(got_garbage)
    end.

appender(Parent, N) ->
    seed(),
    Res = appender_1(N, {}),
    Parent ! {self(),Res}.

appender_1(0, T) -> T;
appender_1(N, T0) ->
    U = rnd_term(),
    T = erlang:append_element(T0, U),
    appender_1(N-1, T).

my_appender(N) ->
    seed(),
    my_appender_1(N, []).

my_appender_1(0, T) ->
    list_to_tuple(lists:reverse(T));
my_appender_1(N, T0) ->
    U = rnd_term(),
    T = [U|T0],
    my_appender_1(N-1, T).

heap_frag_vec(Config) when is_list(Config) ->
    N = 512,
    Self = self(),
    ?line Pid = spawn_link(fun() -> vec_appender(Self, N) end),
    receive
	{Pid,Res} ->
	    ?line Res = my_vec_appender(N);
	Garbage ->
	    io:format("Garbage: ~p\n", [Garbage]),
	    ?line ?t:fail(got_garbage)
    end.

vec_appender(Parent, N) ->
    seed(),
    Res = vec_appender_1(N, vector:new(N, 3.556)),
    Parent ! {self(),Res}.

vec_appender_1(0, T) -> T;
vec_appender_1(N, T0) ->
    U = rnd_term(),
    T = vector:set(N, T0, U),
    vec_appender_1(N-1, T).

my_vec_appender(N) ->
    seed(),
    my_vec_appender_1(N, []).

my_vec_appender_1(0, T) ->
    vector:from_list(T);
my_vec_appender_1(N, T0) ->
    U = rnd_term(),
    T = [U|T0],
    my_vec_appender_1(N-1, T).
    
    
seed() ->
    random:seed(3172, 9815, 20129).

rnd_term() ->
    U0 = random:uniform(),
    B = <<U0/float>>,
    {U0,U0 * 2.5 + 3.14,[U0*2.3,B]}.

